	/***********************************************************************/
	/* blok glowny przetwarzania zapisow holterowskich. (C) PA 15.09.97    */
	/*        OKRESLANIE PARAMETROW SYGNALU EKG WOKOL ZESPOLU QRS          */
	/*      OKRESLANIE PARAMETROW CZASOWYCH ZNALEZIONEGO ZESPOLU QRS       */
	/*       OKRESLANIE CECH MORFOLOGICZNYCH ZNALEZIONEGO ZESPOLU QRS      */
	/*               PRZETWARZANIE ZAPISOW HOLTEROWSKICH EKG               */
	/*                                                                     */
	/***********************************************************************/

/* PROCEDURY ZAWARTE W TYM MODULE:
			void BaseLinEstS(INT_S p, INT_S m)
			INT_S SNREst(INT_S p, INT_S snr[3][3], INT_S m)
			INT_C DetSyncPt(INT_S p)
			INT_S VFibSuspend(void)
			void ShapeS(INT_S pbb) // WERSJA Z FILTRACJA HANNINGA
			void ShapeS(INT_S pbb)  // WERSJA BEZ FILTRACJI
			void HolterProcAll(INT_L QrsMaxNr)
			INT_L PTabInsert(INT_S LenS, INT_L LenD)
			INT_L PTabDelete(INT_S LenS, INT_L LenD)
			INT_S ManualPostProc(INT_L ManTab[MAXMANQRS][2], INT_S ManQrsCnt, INT_L* AutoQrsCntP)
*/

#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#ifdef _WINDOWS
	#include <memory.h>
#endif
#include "holpa.h"
#include "holpaext.h"
#include "holpagdf.h"
#include "holdw.h"
#include "h_sigin.h"

/* DEFINICJE SYMBOLI LOKALNYCH */
#define BLBUF       20 // ~156 ms standardowy bufor estymacji izolinii
#define BLBUFX      60 // ~468 ms przedluzony bufor estymacji izolinii
#define BLSAM        7 // ilosc probek usrednianych do estymacji izolinii
#define VELOCDROP    5 // minimalny spadek predkosci poza zespolem QRS
#define IZOMXSTEP  200 // maksymalna dopuszczalna niezgodnosc estymowanej izolinii
#define BBLENGTH     8 // dlugosc bufora estymacji izolinii [probek]
#define BSAMPLER     6 // wspolczynnik podzialu czestotliwosci probkowania
#define BLIMIT     110 // margines dopuszczalnych wahan izolinii [LSB]
#define QRS_NBR_FOR_HR 12 // ilosc QRS uwzglednianych przy obliczaniu HR
#define DELAYED_QRS 140 // procentowe wydluzenie RR zespolow opoznionych
#define MX_WIN 			16 // polokno poszukiwan maksimum modulu wokol pkt, det.
#define WINDOW   		20 // ilosc probek wokol pkt. detekcji 96 ms
#define WIND1    		 7 // ilosc probek przed pkt. detekcji 96 ms
#define VFIBRR     400 // granica RR dla podejrzenia migotania komor
#define VFIBRRVAR   30 // granica zmiennosci [%] RR dla podejrzenia migotania komor
#define VFIBTSH1   200 // granica iloczynu wspolcz. kszt. dla podejrzenia migotania komor

/* wywolywane procedury zawarte w innych modulach */
extern INT_L QrsDet(INT_L p);   /* w: H_QRSD.CPP */
extern void VFibDet(INT_L p, INT_S mode);  /* w: H_QRSD.CPP */
// extern void STProc(INT_L pbb); /* w: H_ST.CPP */

/* ------------------------------------------------------------------- */



	/**********************************************************************/
	/*         OKRESLANIE PARAMETROW SYGNALU EKG WOKOL ZESPOLU QRS        */
	/*                                                                    */
	/**********************************************************************/

void BaseLinEstS(INT_L p, INT_S m)
/* okreslenie poziomu linii izoelektrycznej ekg (izolinii)              */
/* parametry wejsciowe - p - wskaznik w buforze sygnalu      						*/
/*                       m - 0: praca automatyczna,                     */
/*                           1: reczne wskazanie zespolow QRS           */
/*													 2: reset (zerowanie) zmiennych statycznych */
/* PA 14.07.98	v. 1.08																									*/
	{
	static INT_S estbl[3]={0}; // zestaw estymowanych wartosci izolinii
	static INT_S lastbl[3]={0}; // zestaw poprzednich wartosci izolinii
	INT_S i=0; // zmienna iteracyjna numer probki przed pkt. detekcji
	INT_S j=0; // zmienna iteracyjna wskaznik w buforze wartosci minimalnych
	INT_S k=0; // zmienna iteracyjna numer kanalu
	INT_S v=0; // wartosc predkosci
	INT_S bbuf[BLSAM]={0}; // bufor predkosci punktow wyznaczajacych izolinie
	INT_S nbuf[BLSAM]={0}; // bufor probek wyznaczajacych izolinie
	INT_S vmax=0; // maksymalna wartosc predkosci
	INT_S bmax=0; // maksymalna wartosc bufora
	INT_S pmax=0; // wskaznik do wartosci maksymalnej w buforze
	INT_S pmin=0; // wskaznik do wartosci minimalnej w buforze
	INT_S nmax=0; // wartosc maksymalna w buforze
	INT_S nmin=0; // wartosc minimalna w buforze
	INT_S btmp=0; // tymczasowa wartosc izolinii
	INT_S ctmp=0; // tymczasowa wartosc izolinii (bufor)
	INT_S spc=0;  // znacznik uzycia dlugiego bufora

	if(m==2) // reset (zerowanie) zmiennych statycznych
		{
		for(i=0; i<3; i++)
			{
			estbl[i]=0;
			lastbl[i]=0;
			}
		return;
		}

	if(VFibLock)
		{
		 // blokada adaptacji detektora = VFIB lub zlej jakosci sygnalu
    for(k=0; k<3; k++)
      {
			PTab[CurQrsNr].bl[k]=lastbl[k]; // izolinia identyczna jak poprzednio
				// spodziewana (estbl) izolinia zostaje ta sama
				// ostatnia (lastbl) izolinia zostaje ta sama
      }
		return;
		}

	for(k=0; k<3; k++)
		{
		if(!(Cfg[CurCfg].clchn[k] || Cfg[CurCfg].cfgd[k+1])) 
			// wylaczenie przetwarzania dla kanalow uniewaznionych
			{
			PTab[CurQrsNr].bl[k]=lastbl[k];  
			continue;
			}
		spc=0;
	 // obliczenie modulow predkosci dla punktu 0
		if ((v=(sigb[k][((p+1) & (SIGBLEN-1))]-sigb[k][((p-1) & (SIGBLEN-1))])) < 0)
			bmax=bbuf[0]= -v;
		else
			bmax=bbuf[0]= v;
		pmax=0;
		nbuf[0]=sigb[k][(p & (SIGBLEN-1))];
		for (i=1; i<BLBUF; i++)
			{
	 // obliczenie modulow predkosci dla BLBUF punktow przed punktem detekcji
			if ((v=(sigb[k][((p-i+1) & (SIGBLEN-1))]-sigb[k][((p-i-1) & (SIGBLEN-1))])) < 0)
				v = -v;
	 // ...i wsortowanie ich do bufora
			if (i<BLSAM) // bufor nie jest zapelniony
				{
				if (v>bmax) // zapamietanie maksimum bufora...
					{
					bmax=v;  //...wartosc
					pmax=i;  //...wskaznik
					}
				bbuf[i]=v;
				nbuf[i]=sigb[k][((p-i) & (SIGBLEN-1))];
				}
			else  // bufor jest zapelniony
				{
				if(i==BLSAM)
					vmax=bmax; // maksimum bufora staje sie maksimum globalnym
				if(v > bmax) // biezaca wartosc jest wieksza od maksimum bufora
				// nie wstawiaj do tablicy, ale sprawdz maksimum
					{
					if(v > vmax)
						vmax=v;
					}
				else // biezaca wartosc jest mniejsza od maksimum bufora
				// wsortuj do tablicy
					{
					// wymien z wartoscia maksymalna
					bbuf[pmax]=v;
					nbuf[pmax]=sigb[k][((p-i) & (SIGBLEN-1))];
					// znajdz nowe maksimum w buforze
					bmax=bbuf[(pmax=0)];
					for (j=1; j<BLSAM; j++)
						if(bbuf[j]>bmax)
							bmax=bbuf[(pmax=j)]; //...wartosc i wskaznik
					}
				}
			}
		if(VELOCDROP*bmax > vmax)
			{
			// poza zespolem QRS predkosc nie spada ponizej zalozonej granicy
			// podejrzenie artefaktu lub zlej jakosci sygnalu
			spc=1;
			}
		else
			{
			//zaznaczenie buforze wartosci extremalnych
			nmin=nmax=nbuf[0];
			pmin=pmax=0;
			for (j=1; j<BLSAM; j++)
				if(nbuf[j] > nmax)
					nmax=nbuf[(pmax=j)];
				else if(nbuf[j] < nmin)
					nmin=nbuf[(pmin=j)];
			//usrednianie bufora
			btmp=0;
			for (j=0; j<BLSAM; j++)
				btmp+=nbuf[j];
			btmp-=nbuf[pmin];
			btmp-=nbuf[pmax];
			btmp/=(BLSAM-2);
			if(!m) // reczne wskazanie zespolow QRS nie wykorzystuje estymowanej izolinii
				ctmp=btmp-estbl[k];
			else
				ctmp=0;
			if(ctmp < 0)
				ctmp = -ctmp;
			if((ctmp > IZOMXSTEP) && (CurQrsNr > BLSAM))
				//wykryto rozbieznosc estymowanej i biezacej izolinii
				if(nbuf[pmax] < nbuf[pmin]+2)
				//wykryto przesterowanie
					{
					PTab[CurQrsNr].cl=-5; // zespol wykluczony z klasyfikacji
					PTab[CurQrsNr].bl[k]=lastbl[k]; // izolinia identyczna jak poprzednio
							// spodziewana (estbl) izolinia zostaje ta sama
							// ostatnia (lastbl) izolinia zostaje ta sama
					}
				else
					spc=2; //przeszukaj dluzszy odcinek
			else
				{
				PTab[CurQrsNr].bl[k]=btmp;
				//estymacja izolinii
				// najprostsza liniowa z bezwladnoscia:
				estbl[k]=((3*lastbl[k]+btmp))>>2;
				lastbl[k]=btmp;
				}
			}
		if(spc)
			{
			// przeszukiwanie dluzszego bufora
			for (i=BLBUF; i<BLBUFX; i++)
				{
			// obliczenie modulow predkosci dla pozostalych BLBUFX punktow przed punktem detekcji
				if ((v=(sigb[k][((p-i+1) & (SIGBLEN-1))]-sigb[k][((p-i-1) & (SIGBLEN-1))])) < 0)
					v = -v;
				if(v > bmax) // biezaca wartosc jest wieksza od maksimum bufora
				// nie wstawiaj do tablicy, ale sprawdz maksimum
					{
					if(v > vmax)
					vmax=v;
					}
				else // biezaca wartosc jest mniejsza od maksimum bufora
				// wsortuj do tablicy
					{
					// wymien z wartoscia maksymalna
					bbuf[pmax]=v;
					nbuf[pmax]=sigb[k][((p-i) & (SIGBLEN-1))];
					// znajdz nowe maksimum w buforze
					bmax=bbuf[(pmax=0)];
					for (j=1; j<BLSAM; j++)
						if(bbuf[j]>bmax)
							bmax=bbuf[(pmax=j)]; //...wartosc i wskaznik
					}
				}
			if((VELOCDROP*bmax > vmax) && (CurQrsNr>BLSAM))
				{
			// poza zespolem QRS predkosc nie spada ponizej zalozonej granicy
			// podejrzenie VFIB lub zlej jakosci sygnalu
				PTab[CurQrsNr].cl=-5; // zespol wykluczony z klasyfikacji
				PTab[CurQrsNr].bl[k]=lastbl[k]; // izolinia identyczna jak poprzednio
				// spodziewana (estbl) izolinia zostaje ta sama
				// ostatnia (lastbl) izolinia zostaje ta sama
				}
			else
				{
			//zaznaczenie buforze wartosci extremalnych
				nmin=nmax=nbuf[0];
				pmin=pmax=0;
				for (j=1; j<BLSAM; j++)
					if(nbuf[j] > nmax)
						nmax=nbuf[(pmax=j)];
					else if(nbuf[j] < nmin)
						nmin=nbuf[(pmin=j)];
			//usrednianie bufora
				btmp=0;
				for (j=0; j<BLSAM; j++)
					btmp+=nbuf[j];
				btmp-=nbuf[pmin];
				btmp-=nbuf[pmax];
				btmp/=(BLSAM-2);
				if(!m) // reczne wskazanie zespolow QRS nie wykorzystuje estymowanej izolinii
					ctmp=btmp-estbl[k];
				else
					ctmp=0;
				if(ctmp < 0)
					ctmp = -ctmp;
				if((ctmp > IZOMXSTEP) && (CurQrsNr>BLSAM))				
					//wykryto rozbieznosc estymowanej i biezacej izolinii
					{
					btmp=((3*lastbl[k])+btmp)>>2;
					PTab[CurQrsNr].cl=-5; // zespol wykluczony z klasyfikacji
					PTab[CurQrsNr].bl[k]=btmp; // izolinia zmodyfikowana z bezwladnoscia
					estbl[k]=((3*lastbl[k]+btmp))>>2; // spodziewana (estbl) izolinia uwzglednia trend
					lastbl[k]=btmp;            // ostatnia (lastbl) izolinia przyjmuje nowa wartosc
					}
				else
					{
					PTab[CurQrsNr].bl[k]=btmp;
					 //estymacja izolinii
					estbl[k]=((lastbl[k]+btmp))>>1;
					lastbl[k]=btmp;
					}
				}
			}
    }
	}

INT_S SNREst(INT_L p, INT_S snr[3][4], INT_S m)
/* funkcja SNREst() realizuje estymacja stosunku sygnalu do szumu i  		*/
/* modyfikuje wspolczynniki wagowe udzialu kanalow w detekcji 					*/
/* parametry wejsciowe:	p - wskaznik w buforze sygnalu 									*/
/*          						snr - macierz wartosci snr 											*/
/*                      m - 0: praca automatyczna,                     	*/
/*                          1: reczne wskazanie zespolow QRS           	*/
/*													2: tylko badanie przyrostu szumow						*/
/*													3: reset (zerowanie) zmiennych statycznych  */
/* parametry wyjsciowe: snr - zmodyfikowana macierz wartosci snr 				*/
/*                      SNR - globalny wspolczynnik jakosci sygnalu     */
/*                          uwzgledniajacy klasyfikowane kanaly         */
/* PA 27.07.98 v. 1.08																									*/
	{
	INT_S snrb[3]={0, 0, 0}; // bufor wartosci SNR
	INT_S slev[3]={0, 0, 0}; // bufor poziomu sygnalu
	INT_S nlev[3]={0, 0, 0}; // bufor poziomu szumu
	static INT_S snrh[3][16]={{0, 0, 0}}; // bufor 16 poprzednich wartosci SNR
	static INT_L snrs[3]={0, 0, 0}; // suma/srednia 16 wartosci SNR
	static INT_S snrhp=0; // wskaznik w buforze 16 wartosci SNR
	INT_S tmp=0;  // zmienna tymczasowa
	INT_S w0=0;  // suma wartosci SNR we wszystkich kanalach
	INT_S w[CHANS+1]={0};  // tablica wspolczynnikow wagowych udzialu kanalow w detekcji
	INT_S wk=0; // ilosc czynnych kanalow
	INT_S i=0;  // zmienna iteracyjna / licznik punktow pomiarowych
	INT_S ch=0;  // numer kanalu
	INT_S minrq=0; // znacznik aktualnosci minimum w buforze SNR
	INT_L pbb=0; // wskaznik w buforze sygnalu
//	INT_S pbb0=0; // wskaznik w buforze sygnalu
	static INT_L calls=0; // licznik wywolan

	if(m==3) //reset (zerowanie) zmiennych statycznych 
		{
		for(ch=0; ch<CHANS; ch++)
			{
			snrs[ch]=0;
			for(i=0; i<16; i++)
				snrh[ch][i]=0;
			}
		snrhp=0;
		calls=0;
		return 0;
		}

			/* pobranie probek na odcinku PQ */
	w[CHANS]=0;
	for(ch=0; ch<CHANS; ch++)   // poszukiwanie czynnych kanalow i odczyt SNR
		{
		if(Cfg[CurCfg].cfgd[ch+1])
			{
			pbb=p-12;
			snrb[ch]=0;
			for (i=0; i<4; i++)
				snrb[ch]+=sigb[ch][(pbb+i) & (SIGBLEN-1)];
			snrb[ch]>>=2;
			nlev[ch]=2;
			for (i=0; i<4; i++)
				{
				if((tmp=sigb[ch][((pbb+i) & (SIGBLEN-1))]-snrb[ch]) > 0)
					nlev[ch]+=tmp;
				else
					nlev[ch]-=tmp;
				}
			snr[ch][3]=nlev[ch];
			if(m==2)
				continue;

			/* pobranie probek na odcinku QS */
			pbb=p-2;
			slev[ch]=0;
			for (i=0; i<4; i++)
				{
				if((tmp=sigb[ch][((pbb+i) & (SIGBLEN-1))]-snrb[ch]) > 0)
					slev[ch]+=tmp;
				else
					slev[ch]-=tmp;
				}
			if(!m) //praca ciagla - wykorzystanie bezwladnosci SNR
				{
			/* statystyka snr */
				snrs[ch]-=snrh[ch][snrhp];
				if(snrh[ch][snrhp]==snr[ch][2]) /* bufor opuszcza wartosc minimalna */
					minrq=1;
				snrs[ch]+=(tmp=slev[ch]/nlev[ch]);
				if((snrh[ch][snrhp]=tmp) < snr[ch][2])
					snr[ch][2]=tmp; /* nowa wartosc jest mniejsza niz minimum */
				if (minrq)
					{
					snr[ch][2]=snrh[ch][0];
					for(i=1; i<16; i++)
						if (snr[ch][2] > snrh[ch][i])
							snr[ch][2]=snrh[ch][i]; /* wartosc minimalna z pozostalych wart. bufora */
					minrq=0;
					}
				snr[ch][1]=(INT_S)(snrs[ch]>>4); /* wartosc srednia */
				if(snr[ch][0] > (snr[ch][1]+snr[ch][2])) /* natychmiast w dol */
					snr[ch][0]=((snr[ch][1]+snr[ch][2]) >> 1);
				else if(snr[ch][0] < ((snr[ch][1]+snr[ch][2]) >> 2)) /* powoli w gore */
					snr[ch][0]+=2;
				}
			else
				{
				snr[ch][0]=(slev[ch]/nlev[ch]);
				}
			if(snr[ch][0]>0x007F)
				snr[ch][0]=0x007F;
			w[CHANS]+=(w[ch]=snr[ch][0]);
			wk++;
			}
		} 

	if(m==2)
		{
		return 0;
		}

	snrhp++;
	snrhp &= 0x000F;
			/* zmiana wspolczynnikow wagowych udzialu kanalow w detekcji */
	if(Cfg[CurCfg].cfgd[0] && ((++calls) > 15)  && (!m))
		{
		if(wk=(3*w[CHANS])/(wk<<2))  // minimalny dopuszczalny udzial aktywnego kanalu
			{
			for(ch=0; ch<CHANS; ch++)   // poszukiwanie czynnych kanalow i kontrola zakresu modyfikacji
				if(Cfg[CurCfg].cfgd[ch+1])
					if(w[ch] < wk)
						{
						w[CHANS]+=(wk-w[ch]);
						w[ch]=wk;
						}
			for(ch=0; ch<CHANS; ch++)   // poszukiwanie czynnych kanalow i modyfikacja udzialu
				if(Cfg[CurCfg].cfgd[ch+1])
					Cfg[CurCfg].cfgd[ch+1]=(100*w[ch])/w[CHANS];
			}
		}

	/* obliczenie globalnej jakosci sygnalu */
	tmp=0;
	w0=0;
	for(ch=0; ch<CHANS; ch++)
		if(Cfg[CurCfg].clchn[ch])
			{
			w0+=snr[ch][0];
			tmp++;
			}
	switch(tmp) // korekcja ilosci kanalow
		{
		case 1 : return w0<<1;
		case 2 : return (21*w0)>>4;
		case 3 : return w0;
		default: return -1;
		}
	}

INT_C DetSyncPt(INT_L p)
/* funkcja DetSyncPt() okresla punktu maksymalnego modulu 3D amplitudy  */
/* w zespole QRS w okolicach punktu detekcji p +/-80 ms 								*/
/* parametry wejsciowe: p - wskaznik w buforze sygnalu 									*/
/*          						bl - macierz wartosci izolinii 									*/
/* parametry wyjsciowe: k - polozenie maksimum wzgledem punktu detekcji */
/* PA 17.07.98 v. 1.08																									*/
	{
	INT_L mod=0; // suma kwadratow amplitud sygnalu w uwzglednianych kanalach
	INT_L max=0; // wartosc maksimum amplitudy
	INT_C mxx=0; // wskaznik maksimum amplitudy
	INT_C i=0;   // zmienna iteracyjna
	INT_C delta=3; // poprawka (1ms)
	INT_L tmpp=0;  // tymczasowy wskaznik w buforze sygnalu
	INT_L tmpsig=0; // tymczasowa wartosc sygnalu
	INT_S ModChr[2*MX_WIN]=
	/*{16, 19, 22, 26, 30, 35, 40, 45, 50, 54, 57, 61, 64, 64, 64, 64,
		64, 64, 61, 54, 45, 35, 25, 13, 7, 0, 0, 0, 0, 0, 0, 0}; */
 {16, 19, 22, 26, 30, 35, 40, 45, 50, 54, 57, 61, 64, 64, 64, 64,
	64, 64, 64, 64, 61, 57, 54, 50, 45, 40, 35, 30, 25, 19, 13, 7};
	// funkcja charakterystyki modulu resynchronizacji
  if(Cfg[CurCfg].syncmode && !Cfg[CurCfg].cfgd[Cfg[CurCfg].syncmode])
		return 3; // kanal wg. ktorego ma byc liczony punkt synchronizacji zostal wykluczony z klasyfikacji
	switch(Cfg[CurCfg].syncmode)
		{
		case 0: //modul
			for (i=0; i<2*MX_WIN; i++)
				{
				mod=0;
				if(Cfg[CurCfg].cfgd[1])
					{
					tmpsig=(sigb[0][(tmpp=((p+i-MX_WIN) & (SIGBLEN-1)))]-PTab[CurQrsNr].bl[0]);
				  mod+=tmpsig*tmpsig;
					}
				if(Cfg[CurCfg].cfgd[2])
					{
					tmpsig=(sigb[1][tmpp]-PTab[CurQrsNr].bl[1]);
					mod+=tmpsig*tmpsig;
					}
				if(Cfg[CurCfg].cfgd[3])
					{
					tmpsig=(sigb[2][tmpp]-PTab[CurQrsNr].bl[2]);
					mod+=tmpsig*tmpsig;
					}
				mod=(mod*ModChr[i])>>6;
				if(mod > max)
					{
					max=mod;
					mxx=i-MX_WIN;
					}
				}
			break;
		case 1: //kanal 1
      for (i=0; i<MX_WIN+MX_WIN/2; i++)
				{
				tmpsig=(sigb[0][((p+i-MX_WIN) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[0]);
				mod=tmpsig*tmpsig;
				mod=(mod*ModChr[i])>>6;
				if(mod > max)
					{
					max=mod;
					mxx=i-MX_WIN;
					}
				}
      break;
    case 2: //kanal 2
      for (i=0; i<2*MX_WIN; i++)
				{
				tmpsig=(sigb[1][((p+i-MX_WIN) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[1]);
				mod=tmpsig*tmpsig;
				mod=(mod*ModChr[i])>>6;
				if(mod > max)
					{
					max=mod;
					mxx=i-MX_WIN;
					}
				}
      break;
    case 3: //kanal 3
			for (i=0; i<2*MX_WIN; i++)
				{
				tmpsig=(sigb[2][((p+i-MX_WIN) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[2]);
				mod=tmpsig*tmpsig;
				mod=(mod*ModChr[i])>>6;
				if(mod > max)
					{
					max=mod;
					mxx=i-MX_WIN;
					}
				}
			break;
		default: mxx=0;
		}

	if(Cfg[CurCfg].cfgp[0]) // aproksymacja sygnalu miedzy probkami
		{
		INT_L x1=0; // punkt poprzedzajacy maksimum
		INT_L x2=max; // punkt maksimum
		INT_L x3=0; // punkt nastepujacy po maksimum

		//pobranie punktow x1 i x3, x2=max
		switch(Cfg[CurCfg].syncmode)
			{
			case 0: //modul
	// punkt poprzedni
				tmpp=((p+mxx-1) & (SIGBLEN-1));
				if(Cfg[CurCfg].cfgd[1])
					{
					tmpsig=(sigb[0][tmpp]-PTab[CurQrsNr].bl[0]);
					x1=tmpsig*tmpsig;
					}
				if(Cfg[CurCfg].cfgd[2])
					{
					tmpsig=(sigb[1][tmpp]-PTab[CurQrsNr].bl[1]);
					x1+=tmpsig*tmpsig;
					}
				if(Cfg[CurCfg].cfgd[3])
					{
					tmpsig=(sigb[2][tmpp]-PTab[CurQrsNr].bl[2]);
					x1+=tmpsig*tmpsig;
					}
	// punkt nastepny
				tmpp=((p+mxx+1) & (SIGBLEN-1));
				if(Cfg[CurCfg].cfgd[1])
					{
					tmpsig=(sigb[0][tmpp]-PTab[CurQrsNr].bl[0]);
					x3=tmpsig*tmpsig;
					}
				if(Cfg[CurCfg].cfgd[2])
					{
					tmpsig=(sigb[1][tmpp]-PTab[CurQrsNr].bl[1]);
					x3+=tmpsig*tmpsig;
					}
				if(Cfg[CurCfg].cfgd[3])
					{
					tmpsig=(sigb[2][tmpp]-PTab[CurQrsNr].bl[2]);
					x3+=tmpsig*tmpsig;
					}
				break;
      case 1: //kanal 1
	// punkt poprzedni
				tmpsig=(sigb[0][((p+mxx-1) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[0]);
				x1=tmpsig*tmpsig;
	// punkt nastepny
				tmpsig=(sigb[0][((p+mxx+1) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[0]);
				x3=tmpsig*tmpsig;
				break;
      case 2: //kanal 2
	// punkt poprzedni
				tmpsig=(sigb[1][((p+mxx-1) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[1]);
				x1=tmpsig*tmpsig;
	// punkt nastepny
				tmpsig=(sigb[1][((p+mxx+1) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[1]);
				x3=tmpsig*tmpsig;
				break;
      case 3: //kanal 3
	// punkt poprzedni
				tmpsig=(sigb[2][((p+mxx-1) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[2]);
				x1=tmpsig*tmpsig;
	// punkt nastepny
				tmpsig=(sigb[2][((p+mxx+1) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[2]);
				x3=tmpsig*tmpsig;
				break;
			default: mxx=0;
			}
		if((x1+x3)==(x2<<1))
			delta=3;
		else
			delta=(((3*x1-(x2<<2)+x3)<<2)+((x1-(x2<<1)+x3)>>1))/(x1-(x2<<1)+x3)-5;
		if(delta < 0)
			delta=0;
		else if(delta > 7)
			delta=7;
		// dopuszczalne wartosci delta: 0...7
		// delta =3 oznacza wystapienie maksimum dokladnie w probce x2
		// zaokraglenia poprzez >>3 daja przyblizenie do najblizszej probki
		}
	if((mxx > -MX_WIN) && (mxx < MX_WIN))
		return ((mxx<<3)+delta);
	else
		return 3;
	}
/* ------------------------------------------------------------------- */



	/**********************************************************************/
	/*       OKRESLANIE CECH MORFOLOGICZNYCH ZNALEZIONEGO ZESPOLU QRS     */
	/*      I WYZNACZENIE JEGO REPREZENTACJI DLA POTRZEB KLASYFIKATORA    */
	/**********************************************************************/

INT_S VFibSuspend(void)
/* funkcja VFibSuspend() oblicza stosunek pola powierzchni do obwodu jako  */
/* miare mozliwosci zaistnienia zjawiska migotania komor VFib              */
/* jezeli wartosc mierzona jest wieksza od zadanego progu - funkcja zwraca */
/* wartosc 1, w przeciwnym przypadku - 0.                                  */
/* PA 15.04.98 v.1.08                  					   												 */

	{
	INT_S i=0;           // zmienna iteracyjna

	INT_S tmpv0=0; // wartosc bezwzgledna predkosci
	INT_L se0=0;     // suma wartosci bezwzglednych
	INT_S sv0=0;          // suma wartosci bezwzglednych predkosci
	INT_S tmp0=0;         // biezaca wartosc sygnalu
	INT_S next0=0;        // nastepna wartosc sygnalu

	INT_S tmpv1=0; // wartosci bezwzgledna predkosci
	INT_L se1=0;     // suma wartosci bezwzglednych
	INT_S sv1=0;          // suma wartosci bezwzglednych predkosci
	INT_S tmp1=0;         // biezaca wartosc sygnalu
	INT_S next1=0;        // nastepna wartosc sygnalu

  INT_L wav=0;

  // przygotowanie buforow predkosci
  tmpv0=(next0=PTab[CurQrsNr].wc[1][0])-(tmp0=PTab[CurQrsNr].wc[0][0]);
  tmpv1=(next1=PTab[CurQrsNr].wc[1][1])-(tmp1=PTab[CurQrsNr].wc[0][1]);
//  tmpv0=(next0=(sigb[0][((pbb+1) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[0]))-(tmp0=(sigb[0][((pbb) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[0]));
//  tmpv1=(next1=(sigb[1][((pbb+1) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[1]))-(tmp1=(sigb[1][((pbb) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[1]));

	for(i=2; i<WCLENGTH;i++)
		{
/* kanal 0 */
    if(tmp0 < 0)
      tmp0 = -tmp0;
    se0+=tmp0;      /* sum abs */
    if(tmpv0 < 0)
      tmpv0 = -tmpv0;
    sv0+=tmpv0;        /* sum abs diff */
    tmp0=next0;
    tmpv0=(next0=PTab[CurQrsNr].wc[i][0])-tmp0;
//    tmpv0=(next0=(sigb[0][((pbb+i) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[0]))-tmp0;


/* kanal 1 */
    if(tmp1 < 0)
      tmp1 = -tmp1;
    se1+=tmp1;      /* sum abs */
    if(tmpv1 < 0)
      tmpv1 = -tmpv1;
    sv1+=tmpv1;        /* sum abs diff */
		if((sv0=tmp1-tmp0) <0)
      sv0= -sv0;
    sv1+=sv0;
    tmp1=next1;
    tmpv1=(next1=PTab[CurQrsNr].wc[i][1])-tmp1;
//    tmpv1=(next1=(sigb[1][((pbb+i) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[1]))-tmp1;
		}

	/* obliczenie stosunku pola powierzchni do obwodu */
  se0*=se1;
  sv0*=sv1;
  if(sv0>>=2)
    wav=(se0+(sv0>>1))/sv0;
  else
    wav=se0;
  if(wav > VFIBTSH1)
    return 1;
	else
    return 0;
	}

#ifdef FILTR_SH
void ShapeS(INT_L pbb) // WERSJA Z FILTRACJA HANNINGA
/* funkcja ShapeS() przygotowuje trojwymiarowa znormalizowana reprezentacje */
/* zespolow QRS  			   				    																				*/
/* parametry wejsciowe: pbb - wskaznik w buforze sygnalu 		    						*/
/* PA 28.07.98 v.1.08                  					    												*/
	{
	INT_S i=0; // zmienna iteracyjna (nr probki)
	INT_S c=0; // numer kanalu
	INT_S delta=0;
	INT_S dy0=0; // wspolczynnik korekcji poprzedniego punktu
	INT_S dy1=0; // wspolczynnik korekcji nastepnego punktu
#ifdef SHSUBS
	INT_S SigTable[2*WCLENGTH][3]={{0}}; //tymczasowa tablica sygnalu
#else
	INT_S SigTable[WCLENGTH][3]={{0}}; //tymczasowa tablica sygnalu
#endif
	INT_S SigMx=-32767; //tymczasowe maksimum sygnalu
	INT_S SigMn=32767; //tymczasowe minimum sygnalu
	INT_S SigTmp=0; //tymczasowa wartosc sygnalu
	INT_S MaxAmp=0; //maksimum amplitudy sygnalu
	INT_S Inc=0; //mnoznik amplitudy sygnalu przy normalizacji
	INT_S tmp0=0;  //chwilowa wartosc sygnalu w probce -1
	INT_S tmp1=0;  //chwilowa wartosc sygnalu w probce 0 (poprzedniej)
	INT_S tmp2=0;  //chwilowa wartosc sygnalu w probce 1 (nastepnej)
	INT_S tmp3=0;  //chwilowa wartosc sygnalu w probce +2
	INT_L p0=0;  // indeks w buforze sygnalu 0
	INT_L p1=0;  // indeks w buforze sygnalu 1
	INT_L p2=0;  // indeks w buforze sygnalu 2
	INT_L p3=0;  // indeks w buforze sygnalu 3
	INT_L p4=0;  // indeks w buforze sygnalu 4
	INT_L p5=0;  // indeks w buforze sygnalu 5

	delta=PTab[CurQrsNr].AMax-(PTab[CurQrsNr].AMax>>3);
	if(delta<3)
		pbb--;
	dy1=(delta+5)& 0x0007;
	dy0=8-dy1;
	if(Cfg[CurCfg].cfgc[1]==1) // normalizacja amplitudy
		{
		for (c=0; c<3; c++)
			{
			if(!(Cfg[CurCfg].clchn[c] || Cfg[CurCfg].cfgd[c+1])) 
			// wylaczenie przetwarzania dla kanalow uniewaznionych
				continue;			
			SigMx=-32767;
			SigMn=32767;
			p0=(pbb-2) & (SIGBLEN-1);
			p1=(pbb-1) & (SIGBLEN-1);
			p2=pbb & (SIGBLEN-1);
			p3=(pbb+1) & (SIGBLEN-1);
			p4=(pbb+2) & (SIGBLEN-1);
			p5=(pbb+3) & (SIGBLEN-1);
			tmp0=(((sigb[c][p1]<<1)+sigb[c][p0]+sigb[c][p2])>>2)-PTab[CurQrsNr].bl[c];
			tmp1=(((sigb[c][p2]<<1)+sigb[c][p1]+sigb[c][p3])>>2)-PTab[CurQrsNr].bl[c];
			tmp2=(((sigb[c][p3]<<1)+sigb[c][p2]+sigb[c][p4])>>2)-PTab[CurQrsNr].bl[c];
			tmp3=(((sigb[c][p4]<<1)+sigb[c][p3]+sigb[c][p5])>>2)-PTab[CurQrsNr].bl[c];
#ifdef SHSUBS
			for (i=0; i<2*WCLENGTH; i++)
#else
			for (i=0; i<WCLENGTH; i++)
#endif
				{
				SigTmp=SigTable[i][c]=(((3*tmp1-tmp0)*dy0)+((3*tmp2-tmp3)*dy1))>>5;
				if(SigTmp<SigMn)
					SigMn=SigTmp;
				if(SigTmp>SigMx)
					SigMx=SigTmp;
				tmp0=tmp1;
				tmp1=tmp2;
				tmp2=tmp3;
				tmp3=(((sigb[c][((p5+i) & (SIGBLEN-1))]<<1)+sigb[c][((p4+i) & (SIGBLEN-1))]+sigb[c][((p5+i+1) & (SIGBLEN-1))])>>2)-PTab[CurQrsNr].bl[c];
				}
			if(SigMx>-SigMn)
				MaxAmp=SigMx;
			else
				MaxAmp=-SigMn;
			if(MaxAmp)
				Inc=32767/MaxAmp;
			else
				Inc=256;
#ifdef SHSUBS
			for (i=0; i<2*WCLENGTH; i++)
				PTab[CurQrsNr].wc[i/2][c]=(INT_C)((SigTable[i][c]*Inc)>>8);
#else
			for (i=0; i<WCLENGTH; i++)
				PTab[CurQrsNr].wc[i][c]=(INT_C)((SigTable[i][c]*Inc)>>8);
#endif
			}
		}
	else  // bez normalizacji amplitudy
		{
		for (c=0; c<3; c++)
			{
			if(!(Cfg[CurCfg].clchn[c] || Cfg[CurCfg].cfgd[c+1])) 
			// wylaczenie przetwarzania dla kanalow uniewaznionych
				continue;			
			p0=(pbb-2) & (SIGBLEN-1);
			p1=(pbb-1) & (SIGBLEN-1);
			p2=pbb & (SIGBLEN-1);
			p3=(pbb+1) & (SIGBLEN-1);
			p4=(pbb+2) & (SIGBLEN-1);
			p5=(pbb+3) & (SIGBLEN-1);
			tmp0=(((sigb[c][p1]<<1)+sigb[c][p0]+sigb[c][p2])>>2)-PTab[CurQrsNr].bl[c];
			tmp1=(((sigb[c][p2]<<1)+sigb[c][p1]+sigb[c][p3])>>2)-PTab[CurQrsNr].bl[c];
			tmp2=(((sigb[c][p3]<<1)+sigb[c][p2]+sigb[c][p4])>>2)-PTab[CurQrsNr].bl[c];
			tmp3=(((sigb[c][p4]<<1)+sigb[c][p3]+sigb[c][p5])>>2)-PTab[CurQrsNr].bl[c];
#ifdef SHSUBS
			for (i=0; i<2*WCLENGTH; i++)
				{
				PTab[CurQrsNr].wc[i/2][c]=(INT_C)((((3*tmp1-tmp0)*dy0)+((3*tmp2-tmp3)*dy1))>>8);
#else
			for (i=0; i<WCLENGTH; i++)
				{
				PTab[CurQrsNr].wc[i][c]=(INT_C)((((3*tmp1-tmp0)*dy0)+((3*tmp2-tmp3)*dy1))>>8);
#endif
				tmp0=tmp1;
				tmp1=tmp2;
				tmp2=tmp3;
				tmp3=(((sigb[c][((p5+i) & (SIGBLEN-1))]<<1)+sigb[c][((p4+i) & (SIGBLEN-1))]+sigb[c][((p5+i+1) & (SIGBLEN-1))])>>2)-PTab[CurQrsNr].bl[c];
				}
			}
		}
	}

#else
void ShapeS(INT_L pbb)  // WERSJA BEZ FILTRACJI
/* funkcja ShapeS() przygotowuje trojwymiarowa znormalizowana reprezentacje */
/* zespolow QRS  			   				    																				*/
/* parametry wejsciowe: pbb - wskaznik w buforze sygnalu 		    						*/
/* PA 07.05.98 v.1.08                  					    												*/
  {
  INT_S i=0; // zmienna iteracyjna (nr probki)
  INT_S c=0; // numer kanalu
  INT_S delta=0;
  INT_S dy0=0; // wspolczynnik korekcji poprzedniego punktu
  INT_S dy1=0; // wspolczynnik korekcji nastepnego punktu
#ifdef SHSUBS
  INT_S SigTable[2*WCLENGTH][3]={{0}}; //tymczasowa tablica sygnalu
#else
  INT_S SigTable[WCLENGTH][3]={{0}}; //tymczasowa tablica sygnalu
#endif
  INT_S SigMx=-32767; //tymczasowe maksimum sygnalu
  INT_S SigMn=32767; //tymczasowe minimum sygnalu
  INT_S SigTmp=0; //tymczasowa wartosc sygnalu
  INT_S MaxAmp=0; //maksimum amplitudy sygnalu
	INT_S Inc=0; //mnoznik amplitudy sygnalu przy normalizacji
  INT_S tmp0=0;  //chwilowa wartosc sygnalu w probce -1
	INT_S tmp1=0;  //chwilowa wartosc sygnalu w probce 0 (poprzedniej)
  INT_S tmp2=0;  //chwilowa wartosc sygnalu w probce 1 (nastepnej)
  INT_S tmp3=0;  //chwilowa wartosc sygnalu w probce +2

  delta=PTab[CurQrsNr].AMax-(PTab[CurQrsNr].AMax>>3);
  if(delta<3)
    pbb--;
	dy1=(delta+5)& 0x0007;
  dy0=8-dy1;
	if(Cfg[CurCfg].cfgc[1]==1) // normalizacja amplitudy
		{
    for (c=0; c<3; c++)
      {
			if(!(Cfg[CurCfg].clchan[c] || Cfg[CurCfg].cfgd[c+1])) 
			// wylaczenie przetwarzania dla kanalow uniewaznionych
				continue;			
      SigMx=-32767;
      SigMn=32767;
      tmp0=sigb[c][((pbb-1) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[c];
			tmp1=sigb[c][((pbb) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[c];
      tmp2=sigb[c][((pbb+1) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[c];
      tmp3=sigb[c][((pbb+2) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[c];
#ifdef SHSUBS
      for (i=0; i<2*WCLENGTH; i++)
#else
      for (i=0; i<WCLENGTH; i++)
#endif
				{
				SigTmp=SigTable[i][c]=(((3*tmp1-tmp0)*dy0)+((3*tmp2-tmp3)*dy1))>>5;
				if(SigTmp<SigMn)
					SigMn=SigTmp;
				if(SigTmp>SigMx)
					SigMx=SigTmp;
				tmp0=tmp1;
				tmp1=tmp2;
				tmp2=tmp3;
				tmp3=sigb[c][((pbb+i+3) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[c];
				}
      if(SigMx>-SigMn)
				MaxAmp=SigMx;
			else
				MaxAmp=-SigMn;
      Inc=32767/MaxAmp;
#ifdef SHSUBS
      for (i=0; i<2*WCLENGTH; i++)
				PTab[CurQrsNr].wc[i/2][c]=(INT_C)((SigTable[i][c]*Inc)>>8);
#else
      for (i=0; i<WCLENGTH; i++)
				PTab[CurQrsNr].wc[i][c]=(INT_C)((SigTable[i][c]*Inc)>>8);
#endif
			}
		}
	else  // bez normalizacji amplitudy
		{
		for (c=0; c<3; c++)
			{
			if(!(Cfg[CurCfg].clchan[c] || Cfg[CurCfg].cfgd[c+1])) 
			// wylaczenie przetwarzania dla kanalow uniewaznionych
				continue;			
			tmp0=sigb[c][((pbb-1) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[c];
			tmp1=sigb[c][((pbb) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[c];
			tmp2=sigb[c][((pbb+1) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[c];
			tmp3=sigb[c][((pbb+2) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[c];
#ifdef SHSUBS
			for (i=0; i<2*WCLENGTH; i++)
				{
				PTab[CurQrsNr].wc[i/2][c]=(INT_C)((((3*tmp1-tmp0)*dy0)+((3*tmp2-tmp3)*dy1))>>8);
#else
			for (i=0; i<WCLENGTH; i++)
				{
				PTab[CurQrsNr].wc[i][c]=(INT_C)((((3*tmp1-tmp0)*dy0)+((3*tmp2-tmp3)*dy1))>>8);
#endif
				tmp0=tmp1;
				tmp1=tmp2;
				tmp2=tmp3;
				tmp3=sigb[c][((pbb+i+3) & (SIGBLEN-1))]-PTab[CurQrsNr].bl[c];
				}
			}
		}
	}
#endif
/* ------------------------------------------------------------------- */


	/**********************************************************************/
	/*         AUTOMATYCZNE PRZETWARZANIE ZAPISOW HOLTEROWSKICH EKG       */
	/*                                                                    */
	/**********************************************************************/
 
void HolterProcAll(INT_L QrsMaxNr, INT_L StartPt)
	/* funkcja HolterProcAll() dokonuje odczytu sygnalu holtera z pliku i       */
	/* wywoluje kolejne etapy przetwarzania: 				      											*/
	/*    QrsDet() - detekcja zespolow QRS 				      												*/
	/* 	  BaseLinEst() - estymacja izolinii 				      											*/
	/*    DetMaxMod() - korekcja punktu centrujacego na max amplitudy 	      	*/
	/*    Rr2() - obliczanie interwalu miedzyuderzeniowego i lokalnej wartosci  */
	/*		Heart Rate 						      																					*/
	/*    TimeLoc() - okreslenie typu lokalizacji czasowej (przedwczesnosci itp)*/
	/* 	  SNREst() - okreslenie jakosci sygnalu EKG w kazdym kanale 	      		*/
	/*    Shape() - obliczenie wspolczynnikow ksztaltu (morfologii) 	      		*/
	/*  	STProc() - obliczenie parametrow odcinka ST 			      							*/
	/* PA 25.08.98 v. 1.08 							      																	*/
	{	
	INT_L pb_Start=0; /* poczatek zapisu bufora sygnalu */
  INT_S pb_Switch=0; /* przelacznik polowek bufora sygnalu */
  
  INT_L pbr=0; /* wskaznik w buforze sygnalu przy odczycie */
	INT_L pbc=0; /* numer probki w buforze dla ktorej nastapila pozytywna detekcja QRS */
  INT_L y=0; /* numer probki w pliku dla ktorej nastapila pozytywna detekcja QRS */
	INT_L cp=0; /* pozycja odczytu z pliku danych */
	INT_S NLevel=0; /* globalny poziom szumow */
	INT_S i=0; /* zmienna iteracyjna */
	INT_S c=0; /* zmienna iteracyjna (numer kanalu)*/
	INT_S snr[3][4]={{100}, {100}, {100}};  /* stosunek sygnalu do szumu */
	INT_S anr=0; // skumulowana ilosc artefaktow (zm. kontrolna)
	INT_S rr=0;  // biezacy surowy interwal miedzyuderzeniowy
	INT_S lastrr=0;  // poprzedni surowy interwal miedzyuderzeniowy
	INT_S drr=0; // przyrost rr/ wskaznik rytmu dla detektora VFIB
	INT_L SigPtr=0; // wskaznik w pliku/pamieci sygnalu
	INT_L EofChk=0; // licznik wczytanych probek sygnalu / kontrola wystapienia konca pliku

#ifdef SAVEDETF   // zapis przebiegu funkcji detekcyjnej
	FILE *tmp=NULL; // plik do ktorego zostanie zapisana funkcja detekcyjna	
	tmp=fopen("detfun.res","wt");
#endif

#ifdef EXTSIG_IN // sygnal pobierany z pliku
  /* przygotowanie pliku */
  FILE *ekg=NULL; // plik wejsciowy (hex, LSB-first, chans interlaced)
	if ((ekg = fopen(sigfname, "rb"))  == NULL)
    {
    DetStatus.ErrMsgNr=-1;     // signal file open error
      SendMessage(hKernelWnd, AHM_PROCALL, 0, (LPARAM)&DetStatus);
    return;
    }
  if(SigFmt==3)
    fseek(ekg, 1480808l, SEEK_SET);
  else
		fseek(ekg, 0, SEEK_SET);
#endif

//###############
	FILE *wcf=NULL; // plik do ktorego zostanie zapisana tablica ksztatw
	wcf=fopen("qrsbuff.txt","wt");
//###############
	
// reset (zerowanie) zmiennych statycznych
// izolinii: 
	BaseLinEstS(pbc, 2);
// pomiaru SNR
	SNREst(pbc, snr, 3);
	EofChk=SIGBLEN/2;
  while (EofChk > 0)
		{
		EofChk--;
    if(!(cp & SIGBLEN/2-1)) // co 131073 probek
	    {
			if(pb_Switch)
				{
				pb_Start=SIGBLEN/2; // druga polowa bufora				
				pb_Switch=0;
				}
			else
				{
				pb_Start=0; // pierwsza polowa bufora
				pb_Switch=1;
				}
#ifdef EXTSIG_IN			
			if(cp)			
				EofChk=GetECGFile(ekg, -1, SIGBLEN/2, pb_Start);			
			else	// pierwsza porcja sygnalu
				EofChk=GetECGFile(ekg, cp, SIGBLEN/2, pb_Start);				
#else
			EofChk=GetECGMemory(cp, SIGBLEN/2, pb_Start);
#endif
			}
		/* sprawdzenie czy nie uplynal limit czasowy */
		if((cp) > TimeLimit)
			{
			DetStatus.ErrMsgNr=-4; //time limit reached
			break;
			}

		if(!(cp & SIGBLEN/2-1)) // co 131073 probek
			{
			/* uzupelnienie struktury DetStatus */
			DetStatus.SampleNr=cp;
			DetStatus.CurQrsNr=CurQrsNr;
			DetStatus.Artef_Nr=anr;
			Anuluj_Button= (INT_C)SendMessage(hKernelWnd, AHM_PROCALL, 0, (LPARAM)&DetStatus);

		/* sprawdzenie czy nie nacisnieto klawisza "Anuluj" */
			if(Anuluj_Button)
				{
				DetStatus.ErrMsgNr=-3; //user break
				break;
				}
			}
		if(!(cp & 0x01FF)) // co 512 probek
			{
			if(CurQrsNr)
				{
				if(cp > PTab[CurQrsNr-1].DetPt + 0x01FF)
					{
					SNREst(pbc, snr, 2);  // testowanie wzrostu globalnego poziomu szumow
					if((snr[0][3]+snr[1][3]+snr[2][3])>(NLevel<<1))
						{
						PTab[CurQrsNr].DetPt=cp;
						PTab[CurQrsNr].MorfT=4; // zespol oznaczony jako artefakt 
					/* wykluczenie z klasyfikacji zespolow zakloconych */
						anr++;
						PTab[CurQrsNr].cl=-3;
						for (c=0; c<3; c++)
							for (i=0; i<WCLENGTH; i++)
								PTab[CurQrsNr].wc[i][c]=0;
						CurQrsNr++;
						}
					}
				}
			else if(cp>(StartPt+0x01FF))
				{
				DetStatus.ErrMsgNr=-5; //no QRS detected during first four seconds
				break;
				}
			}
		if(cp>StartPt)
			{
		/* detekcja zespolow qrs */
			if(y=QrsDet(pbr))
				y+=StartPt;
		  if(CurQrsNr) 
				if(y < PTab[CurQrsNr-1].DetPt+EYECLOSING)
					y=0;

		/* jesli odnaleziono zespol qrs */
			if(y)
				{
			/* zapis punktu detekcji */
				if(CurQrsNr == QrsMaxNr)
					{
				//!! DetStatus.ErrMsgNr=-2;   // Memory Overflow
					break;
					}
				PTab[CurQrsNr].DetPt=y;
				PTab[CurQrsNr].MorfT=-1; // zespol oznaczony jako nieznanej morfol.
				pbc=(y & (SIGBLEN-1));
		/* inicjacja poziomow referencyjnych detektora VFIB */
				if(!CurQrsNr)
					VFibDet(pbc, 0);
				else
					{
				/* estymacja izolinii */
					BaseLinEstS(pbc, 0);
				/* korekcja maksimum amplitudy (X-det -> X-max) */
					PTab[CurQrsNr].AMax=DetSyncPt(pbc);
					pbc=((y+(PTab[CurQrsNr].AMax >> 3)) & (SIGBLEN-1));

//##############
	if(CurQrsNr<1000)
		{
		for(SigPtr=y-15; SigPtr<y+15; SigPtr++)
			{
			fprintf(wcf, "%d\n", sigb[1][SigPtr]); ///#
			}
		//fprintf(wcf, "%d\n", PTab[CurQrsNr].bl[1]); ///#
		}
//##############


				/* okreslenie regularnosci rytmu */
					lastrr=rr;
					if(CurQrsNr>DROPQRS)
						rr=((INT_S)(PTab[CurQrsNr].DetPt-PTab[CurQrsNr-2].DetPt)<<3)+(PTab[CurQrsNr].AMax-PTab[CurQrsNr-2].AMax);
					else
						rr=1000; // rr jest podawane w ms
					PTab[CurQrsNr].localRR = rr;
					if(rr <VFIBRR) // jezeli wartosc RR jest mniejsza od progu
						drr=1;
					else if(lastrr >0)
						{
						if((drr=rr-lastrr) <0)
							drr=-drr;
						if((INT_S)((100l*drr)/lastrr)>VFIBRRVAR) // jezeli przyrost jest wiekszy od progu
							drr=1;
						else
							drr=0;
						}

				/* okreslenie jakosci sygnalu */
					if(SNREst(pbc, snr, 0)<Cfg[CurCfg].cfgc[4])
						{
				/* wykluczenie z klasyfikacji zespolow zakloconych */
						anr++;
						PTab[CurQrsNr].cl=-3;
						for (c=0; c<3; c++)
							for (i=0; i<WCLENGTH; i++)
								PTab[CurQrsNr].wc[i][c]=0;
						}
					else
		/* obliczenie wektora cech */
#ifdef SHSUBS
						ShapeS(pbc-2*WIND1);
#else
						ShapeS(pbc-WIND1);
#endif

					PTab[CurQrsNr].snr[0]=(INT_C)snr[0][0];
					PTab[CurQrsNr].snr[1]=(INT_C)snr[1][0];
					PTab[CurQrsNr].snr[2]=(INT_C)snr[2][0];
					NLevel=snr[0][3]+snr[1][3]+snr[2][3]; // okreslenie globalnego poziomu szumow

				/* detekcja czestoskurczu (VFIB detector) */
				// warunki uruchomienia detektora VFIB
					if(PTab[CurQrsNr-1].MorfT==6) //bezwzgledne uruchomienie detektora
						VFibDet(pbc, 3); //kontynuacja migotania komor
					else if(VFibLock == 1)
						VFibDet(pbc, 2); //poszukiwanie poczatku migotania komor
					 //- drugi obieg
					else if (VFibSuspend() && (drr==1))
					// zespol jest komorowy lub
					// rr zmienilo sie lub jest zbyt wysokie
						VFibDet(pbc, 2); //poszukiwanie poczatku migotania komor
					 //- pierwszy obieg
				// uaktualnienie poziomow referencyjnych co 256 zespol QRS
					else if(!(CurQrsNr & 0x00FF))
						VFibDet(pbc, 1);

//				/* obliczanie parametrow odcinka ST z domyslnymi definicjami */
//				STProc(pbc);
					}
				CurQrsNr++;
				}

#ifdef SAVEDETF   // zapis przebiegu funkcji detekcyjnej
			fprintf(tmp, "%d\t", zzz1); ///#
			fprintf(tmp, "%d\n", zzz2); ///#
#endif
///		fprintf(tmp, "%d\t", sigb[0][pbr]); ///#
///		fprintf(tmp, "%d\n", sigb[1][pbr]); ///#
			}
		pbr=(++cp -PREFETCH) & (SIGBLEN-1);
		} 
	PTab[0].DetPt=0x0100-PREFETCH; // modyfikacja umozliwiajaca reczne wstawianie zespolow przed pierwszym znalezionym automatycznie
	printf("artefaktow: %d", anr);


//###############
	fclose(wcf);
//###############

#ifdef SAVEDETF   // zapis przebiegu funkcji detekcyjnej
	fclose(tmp);
#endif
#ifdef EXTSIG_IN			
	fclose(ekg);
#endif
	DetStatus.SampleNr=cp;
	DetStatus.CurQrsNr=CurQrsNr;
	DetStatus.Artef_Nr=anr;
	SendMessage(hKernelWnd, AHM_PROCALL, 1, (LPARAM)&DetStatus);
	} // koniec HolterProcAll
/* ------------------------------------------------------------------- */


	/**********************************************************************/
	/*						OBSLUGA RECZNEGO WSKAZYWANIA ZESPOLOW QRS               */
	/*          do wykorzystania wylacznie w systemie WIN'95              */
	/**********************************************************************/
#ifdef _WINDOWS // WIN95
INT_L PTabInsert(INT_S LenS, INT_L LenD)
/* funkcja PTabInsert umozliwia uporzadkowane uzupelnianie tablicy wynikowej */
/* PTab przez wstawienie LenS elementow znajdujacych sie w wydzielonym       */
/* obszarze do obszaru glownego w porzadku czasowym                          */
/* parametry wejsciowe:                                                      */
/* 		LenS - ilosc zespolow QRS przeznaczonych do wstawienia                 */
/* 		LenD - ilosc zespolow QRS w tablicy PTab przed rozpoczeciem wstawiania */
/* funkcja zwraca - ilosc zespolow QRS w tablicy PTab po wstawianiu          */
/* PA 14.07.98 v. 1.08 						      																		 */
	{
	INT_S S_Cnt=sizeof(QRS_ATTR); // ilosc przesuwanych zespolow QRS w tablicy docelowej
	INT_L CurQrs_S=0; // punkt synchronizacji zespolu QRS w tablicy zrodlowej
	INT_L S_End=LenD; // tymczasowy wskaznik konca bloku
	INT_L QrsCnt=LenD; // liczba zespolow QRS po uzupelnieniu tablicy PTab

	if (QrsCnt >= MAXAUTOQRS)
		// za malo miejsca w obszarze podstawowym do wstawienia bloku
		return LenD;

	LenD--;
	while(LenS)
		{
		CurQrs_S=(PTab[LenS+MAXAUTOQRS-1].DetPt<<3)+PTab[LenS+MAXAUTOQRS-1].AMax;
		// poszukiwanie
		while(((PTab[LenD].DetPt<<3)+PTab[LenD].AMax) > CurQrs_S)
		  LenD--;
		// unikniecie wstawienia zespolu QRS o identycznym punkcie detekcji
		if(((PTab[LenD].DetPt<<3)+PTab[LenD].AMax) < CurQrs_S)
			{
	    // przesuwanie    		
			memmove(&PTab[LenD+LenS+1], &PTab[LenD+1], (S_End-LenD)*S_Cnt);
			// wstawianie
	    memmove(&PTab[LenD+LenS], &PTab[LenS+MAXAUTOQRS-1], S_Cnt);
			// aktualizacja localRR
			PTab[LenD+LenS].localRR=((PTab[LenD+LenS].DetPt-PTab[LenD+LenS-1].DetPt)<<3)+(PTab[LenD+LenS].AMax-PTab[LenD+LenS-1].AMax);
			PTab[LenD+LenS+1].localRR-=PTab[LenD+LenS].localRR;
		  // porzadkowanie
			PTab[LenS+MAXAUTOQRS-1].DetPt=LenD+LenS; // zwraca numer nowego zespolu QRS po wstawieniu
		  S_End=LenD;
			QrsCnt++;
			}
    LenS--;
    }
  return QrsCnt;
  }

INT_L PTabDelete(INT_S LenS, INT_L LenD)
/* funkcja PTabDelete umozliwia uporzadkowane kasowanie tablicy wynikowej 		*/
/* PTab przez usuniecie LenS elementow znajdujacych sie w wydzielonym obszarze*/
/* z obszaru glownego w porzadku czasowym                          						*/
/* parametry wejsciowe:                                                       */
/* 		LenS - ilosc zespolow QRS przeznaczonych do usuniecia                   */
/* 		LenD - ilosc zespolow QRS w tablicy PTab przed rozpoczeciem usuwania    */
/* funkcja zwraca - ilosc zespolow QRS w tablicy PTab po usuwaniu          		*/
/* PA 14.07.98 v. 1.08 						      																			*/
	{
	INT_S S_Cnt=sizeof(QRS_ATTR); // ilosc przesuwanych zespolow QRS w tablicy docelowej
	INT_L CurQrs_S=0; // punkt synchronizacji zespolu QRS w tablicy zrodlowej
	INT_L S_Beg=0; // tymczasowy wskaznik konca bloku
	INT_L PtrS=1;  // wskaznik w liscie elementow przeznaczonych do skasowania
  INT_L PtrL=0;  // wskaznik w tablicy PTab
	INT_L QrsCnt=LenD-LenS; // liczba zespolow QRS po uzupelnieniu tablicy PTab

  if(LenS<1) return LenD;
	CurQrs_S=(PTab[MAXAUTOQRS].DetPt<<3)+PTab[MAXAUTOQRS].AMax;
    // poszukiwanie pierwszego elementu do skasowania
	while(((PTab[PtrL].DetPt<<3)+PTab[PtrL].AMax) < CurQrs_S)
    PtrL++;
  S_Beg=PtrL;   // teraz PtrL wskazuje na element przeznaczony do skasowania
  while(PtrS<LenS)
    {
    CurQrs_S=(PTab[PtrS+MAXAUTOQRS].DetPt<<3)+PTab[PtrS+MAXAUTOQRS].AMax;
    // poszukiwanie kolejnego elementu do skasowania
    while(((PTab[PtrL].DetPt<<3)+PTab[PtrL].AMax) < CurQrs_S)
      PtrL++;
		// teraz PtrL wskazuje na element przeznaczony do skasowania
		// aktualizacja localRR		
		PTab[S_Beg+1].localRR+=PTab[S_Beg+1-PtrS].localRR;
		// przesuwanie
    memmove(&PTab[S_Beg+1-PtrS], &PTab[S_Beg+1], (PtrL-S_Beg)*S_Cnt);
    // porzadkowanie
    S_Beg=PtrL;
    PtrS++;
		}
	// aktualizacja localRR		
	PTab[S_Beg+1].localRR+=PTab[S_Beg+1-PtrS].localRR;
  // przesuniecie bloku nad ostatnim skasowanym elementem
  memmove(&PTab[S_Beg+1-PtrS], &PTab[S_Beg+1], (LenD-S_Beg)*S_Cnt);
  return QrsCnt;
  }

INT_S ManualPostProc(INT_L ManTab[MAXMANQRS][2], INT_S ManQrsCnt, INT_L* AutoQrsCntP)
	/* funkcja ManualPostProc() dokonuje odczytu sygnalu z pliku i obliczenia   */
	/* parametrow w okolicy wskazanego punktu detekcji: 			      						*/
	/* parametry wejsciowe: ManTab - tablica zawierajaca:                       */
	/*    ManTab[n][0] - punkty detekcji wskazane recznie                       */
	/*    ManTab[n][1] - morfologie wskazane recznie lub -1                     */
	/*    ManQrsCnt - ilosc zespolow QRS wstawionych recznie                    */
	/*    AutoQrsCntP - wskaznik na ilosc zespolow QRS w tablicy PTab           */
	/*    funkcja modyfikuje wskazywana zmienna - uaktualnia ilosc zespolow QRS */
	/* inne wykorzystywane funkcje:                                             */
	/* 	  BaseLinEst() - estymacja izolinii 				      											*/
	/*    DetMaxMod() - korekcja punktu centrujacego na max amplitudy 	      	*/
	/* 	  SNREst() - okreslenie jakosci sygnalu EKG w kazdym kanale 	      		*/
	/*    Shape() - obliczenie wspolczynnikow ksztaltu (morfologii) 	      		*/
	/*  	STProc() - obliczenie parametrow odcinka ST 			      							*/
	/* PA 14.07.98 v. 1.08 						      																		*/
	{
	
	INT_S cq=0;  // numer zespolu QRS w tablicy wejsciowej
	INT_S pb0=0; // wskaznik w buforze sygnalu przy zapisie
	//INT_S TmpSValLo=0; // wartosc mlodszego bajtu sygnalu
	INT_S i=0; // zmienna iteracyjna
	INT_S c=0; // zmienna iteracyjna (numer kanalu)
	INT_S y=0x0100-PREFETCH; // numer probki dla ktorej nastapila pozytywna detekcja QRS
	INT_S UnknownMorfCnt=0; // liczba wskazanych zespolow o nieznanej morfologii
	INT_S NewQrs=0; // ilosc faktycznie dolaczonych zespolow QRS
	INT_L SigFPtr=0; // wskaznik odczytu danych w pliku
	INT_S snr[3][4]={{100}, {100}, {100}};  // stosunek sygnalu do szumu
	INT_L AutoQrsCnt=*AutoQrsCntP; // ilosc zespolow QRS znaleziona automatycznie
	INT_L EofChk=0; // licznik wczytanych probek sygnalu / kontrola wystapienia konca pliku

#ifdef EXTSIG_IN // sygnal pobierany z pliku
  /* przygotowanie pliku */
  FILE *ekg=NULL; // plik wejsciowy (hex, LSB-first, chans interlaced)
	if ((ekg = fopen(sigfname, "rb"))  == NULL)
    {
    DetStatus.ErrMsgNr=-1;     // signal file open error
      SendMessage(hKernelWnd, AHM_PROCALL, 0, (LPARAM)&DetStatus);
    return 0;
    }
#endif

  CurQrsNr=MAXAUTOQRS-1; // zapis do wydzielonej gornej czesci tablicy PTab
  for(cq=0; cq<ManQrsCnt; cq++)
    {
		/* dla kazdego zespolu QRS wskazanego w tabeli ManTab */
    SigFPtr=ManTab[cq][0]-y;
    if(SigFPtr<0)
      //wskazanie zbyt blisko poczatku pliku
			{
			DetStatus.ErrMsgNr=-6;     // to close to the begining of signal file
      SendMessage(hKernelWnd, AHM_PROCALL, 0, (LPARAM)&DetStatus);
			return 0;
      }

#ifdef EXTSIG_IN			
		EofChk=GetECGFile(ekg, SigFPtr, 256, 0);			
#else
		EofChk=GetECGMemory(SigFPtr, 256, 0);
#endif
    if(EofChk<256)
      //wskazanie zbyt blisko konca pliku
			{
			DetStatus.ErrMsgNr=-7;     // to close to the end of signal file
      SendMessage(hKernelWnd, AHM_PROCALL, 0, (LPARAM)&DetStatus);
			return 0;
      }

			/* zapis punktu detekcji */
    if(++CurQrsNr == MAXAUTOQRS+MAXMANQRS)
			{
			DetStatus.ErrMsgNr=-2;   // Memory Overflow
      break;
      }
		NewQrs++;
    PTab[CurQrsNr].DetPt=ManTab[cq][0]; // punkt wskazany przez lekarza
    PTab[CurQrsNr].MorfT=-1; // zespol oznaczony jako nieznanej morfol.

    /* estymacja izolinii */
    BaseLinEstS(y, 1);

		/* korekcja maksimum amplitudy (X-det -> X-max) */
		PTab[CurQrsNr].AMax=DetSyncPt(y);
		y+=(PTab[CurQrsNr].AMax >> 3);

		/* okreslenie jakosci sygnalu */
		if(ManTab[cq][1] < 0) // lekarz nie okreslil morfologii
				// zespol QRS bedzie klasyfikowany automatycznie
			{
			UnknownMorfCnt++;
			PTab[CurQrsNr].MorfT=-1;
			if(SNREst(y, snr, 1)<Cfg[CurCfg].cfgc[4])
				{
	/* wykluczenie z klasyfikacji zespolow zakloconych */
				PTab[CurQrsNr].cl=-3;
				for (c=0; c<3; c++)
					for (i=0; i<WCLENGTH; i++)
						PTab[CurQrsNr].wc[i][c]=0;
				}
			else
	/* obliczenie wektora cech */
				ShapeS(y-WIND1);

			PTab[CurQrsNr].snr[0]=(INT_C)snr[0][0];
			PTab[CurQrsNr].snr[1]=(INT_C)snr[1][0];
			PTab[CurQrsNr].snr[2]=(INT_C)snr[2][0];
			}
		else // lekarz okreslil morfologie, zespol jest wykluczony z klasyfikacji
			{
			PTab[CurQrsNr].MorfT=(INT_C)ManTab[cq][1];
			PTab[CurQrsNr].cl=-8;
			}
//		/* obliczanie parametrow odcinka ST z domyslnymi definicjami */
//		STProc(y);
		}
	*AutoQrsCntP=PTabInsert(NewQrs, AutoQrsCnt);
	for(cq=0; cq<NewQrs; cq++)
		ManTab[cq][1]=PTab[MAXAUTOQRS+cq].DetPt; // numer wstawianego zespolu QRS po wstawieniu

#ifdef EXTSIG_IN			
	fclose(ekg);
#endif

  return UnknownMorfCnt;
  } // koniec ManualFind
#endif // WIN95
/* ------------------------------------------------------------------- */


